@@ -1,6 +1,6 @@
 # $Author: ddumont $
-# $Date: 2008-09-23 13:03:10 +0200 (Tue, 23 Sep 2008) $
-# $Revision: 759 $
+# $Date: 2008-09-29 13:43:04 +0200 (Mon, 29 Sep 2008) $
+# $Revision: 765 $
 
 #    Copyright (c) 2005-2008 Dominique Dumont.
 #
@@ -34,7 +34,7 @@ use Config::Model::Instance ;
 # this class holds the version number of the package
 use vars qw($VERSION @status @level @experience_list %experience_index) ;
 
-$VERSION = '0.627';
+$VERSION = '0.628';
 
 =head1 NAME
 
@@ -440,13 +440,16 @@ $check{experience}=\%experience_index ;
 
 =item generated_by
 
-Mention with a descriptive string if this class was generated by a program.
-This parameter is currently reserved for a model editor (yet to be written).
+Mention with a descriptive string if this class was generated by a
+program.  This parameter is currently reserved for
+L<Config::Model::Itself> model editor.
 
 =cut
 
-my @legal_params = qw/experience status description element level config_dir 
-                      generated_by class_description/;
+my @legal_params = qw/experience status description element level
+                      config_dir generated_by class_description
+                      read_config read_config_dir write_config
+                      write_config_dir/;
 
 sub create_config_class {
     my $self=shift ;
